; CSTART.ASM  -  C startup-code for SIM68K
;
; *******************************************************************************
;
; Updated to run on both the 68k-MBC board and the SIM68k (by J4F)
;
; CREDITS: Thanks to Fred Bayer that suggested me about his implementation for 
;          the Sirichote Motorola 68008 Microprocessor Kit with runtime auto-switch
;
; NOTE: This is for the 1024KB RAM configuration of the 68k-MBC
;
; *******************************************************************************


; Section definitions and memory layout

; IOS equates (68k-MBC)
IOBASE      EQU     $FFFFC                      ; Address base for the I/O ports
EXCWR_PORT  EQU     IOBASE+0                    ; Address of the EXECUTE WRITE OPCODE write port
EXCRD_PORT  EQU     IOBASE+0                    ; Address of the EXECUTE READ OPCODE read port
STOPC_PORT  EQU     IOBASE+1                    ; Address of the STORE OPCODE write port
SER1RX_PORT EQU     IOBASE+1                    ; Address of the SERIAL 1 RX read port 
SYSFLG_PORT EQU     IOBASE+2                    ; Address of the SYSFLAGS read port
SER2RX_PORT EQU     IOBASE+3                    ; Address of the SERIAL 2 RX read port
USRLED_OPC  EQU     $00                         ; USER LED opcode
SER1TX_OPC  EQU     $01                         ; SERIAL 1 TX opcode
SELDISK_OPC EQU     $09                         ; SELDISK opcode
SELTRCK_OPC EQU     $0A                         ; SELTRACK opcode
SELSECT_OPC EQU     $0B                         ; SELSECT opcode
WRTSECT_OPC EQU     $0C                         ; WRITESECT opcode
SER2TX_OPC  EQU     $10                         ; SERIAL 2 TX opcode
ERRDSK_OPC  EQU     $85                         ; ERRDISK opcode
RDSECT_OPC  EQU     $86                         ; READSECT opcode
SDMOUNT_OPC EQU     $87                         ; SDMOUNT opcode
FLUSHBF_OPC EQU     $88                         ; FLUSHBUFF opcode

; Runtime auto-switch definitions
magic_val  equ         $1000           			; If this value is stored...
magic_addr equ         $2              			; ...at this address, we run on the 68k-MBC

lomem      equ         $400            			; Lowest usable address
himem      equ         $FBFF0          			; Highest memory address + 1 (68k-MBC with 1024KB RAM)
stklen     equ         $1000           			; Default stacksize

           option      S0              			; write program ID to S0 record

           section     code
           org         lomem
code       equ         *               			; start address of code section (instructions in ROM)

           section     const
const      equ         *               			; start address of const section (initialized data in ROM)

           section     data
data       equ         *               			; start address of data section (initialized global data ROM/RAM))

           section     bss
bss        equ         *               			; start address of bss section (uninitialized global data in RAM)

           section     heap
heap       equ         *               			; start address of heap section (start of free memory, RAM)

           section     code
start:
           lea         bss,A0
           clr.b       (A0)+           			; set bss section to zero
           cmp.l       #heap,A0
           bcs.s       *-8
           move.l      #-1,__ungetbuf  			; unget-buffer for keyboard input
           move.l      #0,__allocp     			; pointer to allocated memory for malloc-function
           move.l      #heap,__heap    			; pointer to free memory
           move.l      #(himem-stklen),__stack 	; top of stack (for stack overflow detection)
           move.l      #himem,A7       			; initial stack pointer
           jsr         _main

; ...............................................................		   
		   	   
           xdef        __exit
__exit:                                			; exit program
		   cmpi.w      #magic_val,magic_addr.W
           bne.s       exit_sim		   			; jump if running on simulator
		   jmp		   __exit

exit_sim:		   
           trap        #15
           dc.w        0
           bra         start           			; restart

; ...............................................................		   
	   
           xdef        __putch
__putch:                               			; basic character output routine
		   move.l      4(A7),D0
		   ; the char to print is now in d0.b
		   cmpi.w      #magic_val,magic_addr.W
           bne.s       putch_sim				; Jump if running on simulator
		   move.b  	   #SER1TX_OPC,(STOPC_PORT).l	; Write SERIAL 1 TX opcode to IOS
		   move.b      d0,(EXCWR_PORT).l        ; Print current char
		   rts

putch_sim:
           trap        #15
           dc.w        1               			; IDE68K system call 1 -> PUTCH
           rts

; ...............................................................		   
		   
           xdef        __getch
__getch:                               			; Basic character input routine
		   cmpi.w      #magic_val,magic_addr.W
           bne.s       getch_sim				; Jump if running on simulator
		   bsr     	   __kbhit                	; Check if there is an input pending
		   tst.b   	   d0
		   beq     	   __getch                  ; No, wait an input
		   move.b  	   (SER1RX_PORT).l,d0       ; Yes, read Serial port 1 data into D0.B
		   ext.w       d0
           ext.l       d0                      	; D0.L is char, sign extended to 32 bits
		   rts

getch_sim:		   
           trap        #15
           dc.w        3               			; IDE68K system call 3 -> GETCH
           ext.w       D0
           ext.l       D0              			; D0.L is char, sign extended to 32 bits
           rts

; ...............................................................		   
		   
           xdef        __kbhit
__kbhit:
		   cmpi.w      #magic_val,magic_addr.W
           bne.s       kbhit_sim				; Jump if running on simulator
		   move.b  	   (SYSFLG_PORT).l,d0       ; D0.B = SYSFLAG
		   btst    	   #2,d0                    ; Serial 1 Rx char ready bit (bit 2) = 1?
		   bne     	   charRdy                  ; Yes, jump
		   clr.b   	   d0						; No, return with D0.B = 0x00
		   rts
charRdy:
		   move.b  	   #$ff,d0                  ; Yes, D0.B = 0xFF
		   rts

kbhit_sim:
           trap        #15
           dc.w        4               			; IDE68K system call 4 -> KBHIT
           sne         D0              			; D0.B = $FF if char in buffer, $00 if not
           rts

; ...............................................................		   
		    
           xdef        __time
__time:
		   cmpi.w      #magic_val,magic_addr.W
           bne.s       time_sim					; Jump if running on simulator
		   clr.w       d0 						; Not implemented on the 68k-MBC
		   rts

time_sim:		   
           trap        #15             			; IDE68K system call 40 -> GETTIME
           dc.w        40              			; D0 = seconds since Jan 1, 1970, 00:00:00 hr UTC
           rts

; ...............................................................		   
		   
           xdef        __localoffset
		   
__localoffset:
		   cmpi.w      #magic_val,magic_addr.W
           bne.s       locoff_sim				; Jump if running on simulator
		   clr.w       d0 						; Not implemented on the 68k-MBC
		   rts

locoff_sim:		   
           trap        #15             			; IDE68K system call 41 -> LOCALOFFSET
           dc.w        41              			; D0 = offset in seconds between UTC and local time
           rts

; ...............................................................		   
		   
           xdef        stackoverflow
stackoverflow:
           lea         himem,A7        			; reset stackpointer
           lea         stackmsg,A0
           trap        #15             			; print message
           dc.w        7
           bra         __exit          			; abort program

           section     const
stackmsg:
           dc.b        'Stack overflow!',$0A,$0D
           dc.b        'Program aborted',$0A,$0D,0

           section     bss
           xdef        __ungetbuf
__ungetbuf:
           ds.l        1               			; ungetbuffer for stdio functions
           xdef        __allocp
__allocp:
           ds.l        1               			; start of free memory block list
           xdef        __heap
__heap:
           ds.l        1               			; begin of free memory
           xdef        __stack
__stack:
           ds.l        1               			; bottom of stack