/*
 * 68kmbcglue.S -- 68k-MBC board support.
 *
 * Copyright (c) 1995, 1996 Cygnus Support
 * Copyright (c) 2017, Jeff Tranter <tranter@pobox.com>
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 * 
 *              * * * CHANGES FOR THE 68k-MBC (J4F) * * *
 *
 */

#include "asm.h"

	.title "68kmbcglue.S for 68k-MBC"

* IOS equates
IOBASE      =     0xFFFFC               /* Address base for the I/O ports */
EXCWR_PORT  =     IOBASE+0              /* Address of the EXECUTE WRITE OPCODE write port */
EXCRD_PORT  =     IOBASE+0              /* Address of the EXECUTE READ OPCODE read port */
STOPC_PORT  =     IOBASE+1              /* Address of the STORE OPCODE write port */
SER1RX_PORT =     IOBASE+1              /* Address of the SERIAL 1 RX read port */
SYSFLG_PORT =     IOBASE+2              /* Address of the SYSFLAGS read port */
SER2RX_PORT =     IOBASE+3              /* Address of the SERIAL 2 RX read port */
USRLED_OPC  =     0x00                  /* USER LED opcode */
SER1TX_OPC  =     0x01                  /* SERIAL 1 TX opcode */
SETIRQ_OPC  =     0x02                  /* SETIRQ opcode */
SELDISK_OPC =     0x09                  /* SELDISK opcode */
SELTRCK_OPC =     0x0A                  /* SELTRACK opcode */
SELSECT_OPC =     0x0B                  /* SELSECT opcode */
WRTSECT_OPC =     0x0C                  /* WRITESECT opcode */
SER2TX_OPC  =     0x10                  /* SERIAL 2 TX opcode */
ERRDSK_OPC  =     0x85                  /* ERRDISK opcode */
RDSECT_OPC  =     0x86                  /* READSECT opcode */
SDMOUNT_OPC =     0x87                  /* SDMOUNT opcode */
FLUSHBF_OPC =     0x88                  /* FLUSHBUFF opcode */



	.text
	.global SYM (_exit)
	.global SYM (outln)
	.global SYM (outbyte)
	.global SYM (inbyte)
	.global SYM (havebyte)

/*
 * _exit -- Exit from the application. Normally we cause a user trap
 *          to return to the ROM monitor for another run.
 */
	.text
	.align	2
SYM (_exit):
    stop    #0x2700                  // Just stop the CPU

/*
 * inbyte -- get a byte from the console serial port
 *	d0 - contains the byte read in
 *
 */
	.text
	.align	2
SYM (inbyte):
    move.b  (SYSFLG_PORT).l,d0      // D0.B = SYSFLAGS IOS register
    btst    #2,d0                   // Rx char ready? (bit2 = 1?)
    beq     inbyte                  // No, jump (wait for a char)
    move.b  (SER1RX_PORT).l,d0      // Yes, read a char from serial 1
	extw	d0			            // Extend byte result to word
	extl	d0			            // ...and longword
	cmp	    #'\r',d0		        // Is it newline (CR)?
	bne	    notnl1			        // Branch if not
    bsr	    outbyte			        // Echo it
	move.b	#'\n',d0		        // Convert to newline (LF)

notnl1: 
    bsr	    outbyte			        // Echo it
	rts

/*
 * outbyte -- sends a byte out the serial port
 *	d0 - contains the byte to be sent
 *
 */
	.text
	.align	2
SYM (outbyte):
    move.b  #SER1TX_OPC,(STOPC_PORT).l  // Write SERIAL 1 TX opcode to IOS
    move.b  d0,(EXCWR_PORT).l           // Print current char
	rts

/*
 * outln -- sends a string of bytes out the serial port with a CR/LF
 *	a0 - contains the address of the string's first byte
 *	a1 - contains the address of the string's last byte
 */
	.text
	.align	2
SYM (outln):
  	movem.l	d0,-(sp)	                // Save modified registers
    bsr     outstr                      // Print string
    move.b  #'\r',d0		            // Send CR
    bsr     outbyte
    move.b  #'\n',d0		            // Send LF
    bsr     outbyte
	movem.l	(sp)+,d0	                // Restore registers
	rts

/*
 * outstr -- sends a string of bytes out the serial port without a CR/LF
 *	a0 - contains the address of the string's first byte
 *	a1 - contains the address of the string's last byte
 */
	.text
	.align	2
SYM (outstr):
	movem.l	d0,-(sp)	                // Save modified registers
outstr1:    
    move.b  (a0)+,d0                    // D0.B = current char to print
    move.b  #SER1TX_OPC,(STOPC_PORT).l  // Write SERIAL 1 TX opcode to IOS
    move.b  d0,(EXCWR_PORT).l           // Print current char
    cmp.l   a0,a1                       // Last char?
    bne     outstr1                     // No, jump
	movem.l	(sp)+,d0	                // Restore registers
	rts

/*
 * havebyte -- checks to see if there is a byte in the serial port,
 *             returns 1 if there is a byte, 0 otherwise.
 */
	.text
	.align	2
SYM (havebyte):
    move.b  (SYSFLG_PORT).l,d0          // D0.B = SYSFLAGS IOS register
    btst    #2,d0                       // Rx char ready? (bit2 = 1?)
    beq     RXNOTREADY                  // No, jump
	move.l	#1,d0			            // Return 1 to indicate data is available
	rts				                    // Return
RXNOTREADY:
	move.l	#0,d0			            // Return 0 to indicate no data available
	rts
