#!/bin/bash

# *************************************************************************************************
# User definitions.
#
# Adapt to your system if required:
#
#    DOCKER_IMAGE     is the complete docker image name to use
#    SERIAL_USB       is the name of the serial-USB adapter for the automatic upload
#    TERM_COLOR       is the color setting for minicom (on/off)
# *************************************************************************************************
DOCKER_IMAGE=just4fun4just/68k-mbc:v5
SERIAL_USB=/dev/ttyUSB0
TERM_COLOR=off

#
# Others definitions
#
OUT_FILE_ELF=a.out
OUT_FILE=out.sr
OUT_FILE_CRLF=out_win.sr
SRC_DIR=$(pwd)

#
# Delete previous ouput files
#
rm -f $OUT_FILE_CRLF
rm -f $OUT_FILE
rm -f $OUT_FILE_ELF

#
# Ignore the extension if present
#
filename=$1
filename="${filename%.*}"

#
# Check if the file exist. If ok go further, else abort
#
if test -f "$filename.c"; 
then

    #
    # Call docker image to cross-compile
    #
    echo " **** Working directory:" $SRC_DIR
    echo " **** Serial port:" $SERIAL_USB
    echo " **** Docker image:" $DOCKER_IMAGE
    echo " **** Terminal emulator colors:" $TERM_COLOR
    docker run --rm -v "$SRC_DIR":/opt/work $DOCKER_IMAGE /opt/work/slave.sh $filename

    #
    # Upload into the target only if cross-compiling was successful
    #
    if test -f "$OUT_FILE_ELF"; then
        echo " **** Uploading to the target..."
        minicom -w -c $TERM_COLOR -S minicom.txt -D $SERIAL_USB
        echo " **** All done!"
    fi
else
    echo " **** File not found:" $filename.c
fi
