//
// Set a QR203 Thermal Printer on Serial 2 to use compressed font
// to maximize the number of columns
//
//
// Link to the QR203 printer:
//
// https://www.aliexpress.com/item/1005001966687361.html?spm=a2g0s.8937460.0.0.2d442e0e5jSOpY&fbclid=IwAR0c6JS9iYvzhA-cCFUdfYasAq-1C9zpYoH5gBxEUGjLz8uBbQLuopC8k7A
//

#include <stdio.h>

#define SER2TX_OPC  0x10                        // SERIAL 2 TX opcode
#define ESC         0x1B

void printString2(const char *s);
void outch2(volatile char c);
void iosStoreOpc(unsigned char opcode);
void iosExecWriteOpc(unsigned char iodata);

int main(void)
{
    setvbuf(stdout, NULL, _IONBF, 0);           // Disable buffering on stdout
    printf("QR203 thermal printer set @ 42 columns\n\r");
    printString2("QR203 thermal printer\r\n\n");

    // Set Small font (42 columns) + Emphasized mode
    outch2(ESC);
    outch2('!');
    outch2(0b00000101);

    printString2("QR203 thermal printer: Emphasized on\r\n\n");

    // Set bold on
    outch2(ESC);
    outch2('E');
    outch2(1);

    printString2("QR203 thermal printer: Bold on\r\n");
    printString2("123456789+123456789+123456789+123456789+12\r\n");
}

// Print a string on serial 2.
void printString2(const char *s) 
{
    while (*s != 0) 
    {
        outch2(*s);
        s++;
    }
}

// Print a character on serial 2
void outch2(volatile char c) 
{
    iosStoreOpc(SER2TX_OPC);
    iosExecWriteOpc(c);
}

// ---------------------------------------------------------------------------------------------------------------------
//
// IOS low level interface functions
//
// ---------------------------------------------------------------------------------------------------------------------

// IOS Store Opcode
void iosStoreOpc(unsigned char opcode)
{
    // https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf 
    // par. 6.41  Assembler Instructions with C Expression Operands
    asm("move.b %0,(0xFFFFD).l\n\t"        // move.b  <opcode>,(STOPC_PORT).l           ; Write <opcode> to STOPC_PORT
        : : "r" (opcode)); 
}

// IOS Exec Write Opcode
void iosExecWriteOpc(unsigned char iodata)
{
    // https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf 
    // par. 6.41  Assembler Instructions with C Expression Operands
    asm("move.b %0,(0xFFFFC).l\n\t"        // move.b  <iodata>,(EXCWR_PORT).l           ; Write <iodata> to EXCWR_PORT
        : : "r" (iodata)); 
}

