//
// QR203 Thermal Printer on Serial 2.
// <ESC ! n> command test.
//
//
// Link to the QR203 printer:
//
// https://www.aliexpress.com/item/1005001966687361.html?spm=a2g0s.8937460.0.0.2d442e0e5jSOpY&fbclid=IwAR0c6JS9iYvzhA-cCFUdfYasAq-1C9zpYoH5gBxEUGjLz8uBbQLuopC8k7A
//

#include <stdio.h>

#define SER2TX_OPC  0x10                        // SERIAL 2 TX opcode
#define ESC         0x1B

void printString2(const char *s);
void outch2(volatile char c);
void iosStoreOpc(unsigned char opcode);
void iosExecWriteOpc(unsigned char iodata);

int main(void)
{
    setvbuf(stdout, NULL, _IONBF, 0);           // Disable buffering on stdout
    printf("QR203 Thermal printer\n\r");
    printf("<ESC ! n> command test\n\r");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b00000000);
    printString2("QR203 Thermal printer\r\n\n");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b00000001);
    printString2("QR203 Thermal printer\r\n\n");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b00000010);
    printString2("QR203 Thermal printer\r\n\n");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b00000100);
    printString2("QR203 Thermal printer\r\n\n");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b00001000);
    printString2("QR203 Thermal printer\r\n\n");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b00010000);
    printString2("QR203 Thermal printer\r\n\n");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b00100000);
    printString2("QR203 Thermal printer\r\n\n");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b01000000);
    printString2("QR203 Thermal printer\r\n\n");

    outch2((char) ESC);
    outch2('!');
    outch2((char) 0b10000000);
    printString2("QR203 Thermal printer\r\n\n");
   
}

// Print a string on serial 2.
void printString2(const char *s) 
{
    while (*s != 0) 
    {
        outch2(*s);
        s++;
    }
}

// Print a character on serial 2
void outch2(volatile char c) 
{
    iosStoreOpc(SER2TX_OPC);
    iosExecWriteOpc(c);
}

// ---------------------------------------------------------------------------------------------------------------------
//
// IOS low level interface functions
//
// ---------------------------------------------------------------------------------------------------------------------

// IOS Store Opcode
void iosStoreOpc(unsigned char opcode)
{
    // https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf 
    // par. 6.41  Assembler Instructions with C Expression Operands
    asm("move.b %0,(0xFFFFD).l\n\t"        // move.b  <opcode>,(STOPC_PORT).l           ; Write <opcode> to STOPC_PORT
        : : "r" (opcode)); 
}

// IOS Exec Write Opcode
void iosExecWriteOpc(unsigned char iodata)
{
    // https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf 
    // par. 6.41  Assembler Instructions with C Expression Operands
    asm("move.b %0,(0xFFFFC).l\n\t"        // move.b  <iodata>,(EXCWR_PORT).l           ; Write <iodata> to EXCWR_PORT
        : : "r" (iodata)); 
}

