//
// Blink User led until User key is pressed.
// (Specifying registers for local variables with assembler instructions).
//

#include <stdio.h>
void userLed(const char c);
char delay(long loop);
char userKey();

int main(void)
{
    char i,j = 0;
    i = 1;

    //
    // Disable buffering on stdout to avoid printf not flushing unless a newline (LF) is sent.
    // For more info: 
    // https://stackoverflow.com/questions/1716296/why-does-printf-not-flush-after-the-call-unless-a-newline-is-in-the-format-strin
    //
    setvbuf(stdout, NULL, _IONBF, 0);

    printf("Blink User led until User key is pressed\n\n\r");
    printf("Press User key to stop blinking... ");

    while (!j)
    {
        userLed(i);                         // Toggle user led
        j = delay(20000);
        i=!i;
    }
    userLed(0);                             // User led off
    printf("User Key pressed.");
}


// Turn User led on/off using the IOS Opcodes
void userLed(volatile char c) 
{
    //
    // Here we are relying on the fact that gcc puts the passed parameter in register D0.
    // This is generally an unsafe way to do this.
    // Better way is specifying registers for local variables (https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf par. 6.44.2)
    // or using assembler instructions with C expression operands (https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf par. 6.41)
    //
    asm("move.b #00,(0xFFFFD).l\n\t"        // move.b  #USERLED_OPC,(STOPC_PORT).l  ; Write USER LED opcode to IOS
        "move.b %d0,(0xFFFFC).l\n\t");      // move.b  d0,(EXCWR_PORT ).l           ; Send current char value
}

// Delay using a loop of NOP
char delay(long loop)
{
    long i;
    char j = 0;
    for (i=0; i<loop; i++)
    {
        if (userKey() > 0) j = 1;
    }
    return j;
}

// Check if the User key is pressed
char userKey()
{
    register char key asm ("%d0");          // https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf 
                                            //  par. 6.44.2 Specifying Registers for Local Variables

    asm("move.b #0x80,(0xFFFFD).l\n\t"      // move.b  #USERKEY_OPC,(STOPC_PORT).l  ; Write USER KEY opcode to IOS
        "move.b (0xFFFFC).l,%d0");          // move.b  d0,(EXCWR_PORT ).l           ; Read current User Key value
    return key;
}
