//
// Blink User led until User key is pressed.
// (Using assembler instructions with C expression operands).
//

#include <stdio.h>
void userLed(const char c);
char delay(long loop);
char userKey();

int main(void)
{
    char i, j = 0;
    i = 1;

    //
    // Disable buffering on stdout to avoid printf not flushing unless a newline (LF) is sent.
    // For more info: 
    // https://stackoverflow.com/questions/1716296/why-does-printf-not-flush-after-the-call-unless-a-newline-is-in-the-format-strin
    //
    setvbuf(stdout, NULL, _IONBF, 0);

    printf("Blink User led until User key is pressed\n\n\r");
    printf("Press User key to stop blinking... ");
    while (!j)
    {
        userLed(i);                         // Toggle user led
        j = delay(40000);
        i=!i;
    }
    userLed(0);                             // User led off
    printf("User Key pressed.");
}


// Turn User led on/off using the IOS Opcodes
void userLed(char c) 
{
    // https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf 
    // par. 6.41  Assembler Instructions with C Expression Operands
    asm("move.b #00,(0xFFFFD).l\n\t"        // move.b  #USERLED_OPC,(STOPC_PORT).l  ; Write USER LED opcode to IOS
        "move.b %0,(0xFFFFC).l\n\t"         // move.b  <c>,(EXCWR_PORT ).l          ; Send current char value
        : : "r" (c));                                
}

// Delay using a loop of NOP
char delay(long loop)
{
    long i;
    char j = 0;
    for (i=0; i<loop; i++)
    {
        if (userKey() > 0) j = 1;
    }
    return j;
}

// Read User Key status
char userKey()
{
    char key;

    // https://gcc.gnu.org/onlinedocs/gcc-4.8.5/gcc.pdf 
    // par. 6.41  Assembler Instructions with C Expression Operands
    asm("move.b #0x80,(0xFFFFD).l\n\t"     // move.b  #USERKEY_OPC,(STOPC_PORT).l  ; Write USER KEY opcode to IOS
        "move.b (0xFFFFC).l,%0\n\t"        // move.b  <key>,(EXCWR_PORT ).l        ; Read current User Key value
        : "=r" (key));


    return key;
}
