#!/bin/bash

# *******************************************************************************************
# User definitions. 
#
# Select the linker script (LINKER_SCRIPT) to be used accordingly with the installed RAM on 
# your 68k-MBC:
#
#   128KB -> 68kMBC_128_elf.ld
#   512KB -> 68kMBC_512_elf.ld
#  1024KB -> 68kMBC_1024_elf.ld
#
# NOTE: the linker script must be in the same working directory.
# *******************************************************************************************
LINKER_SCRIPT=68kMBC_128_elf.ld

#
# Cross compile from the running docker container
#
echo " **** Compiling: "$1.c
echo " **** Linker script file:" $LINKER_SCRIPT
cd /opt/work
set -o xtrace
/root/.local/bin/m68k-elf-gcc -Wall -O $1.c -o a.out -T $LINKER_SCRIPT -Xlinker -Map=$1.map
exitcode=$?
set +o xtrace
if test $exitcode -eq 0
then
    echo " **** Compiled with no errors"

    /root/.local/bin/m68k-elf-objcopy a.out -O srec $1.sr

    echo " **** Output file (elf format): "a.out
    /root/.local/bin/m68k-elf-objcopy a.out -O binary autoboot.bin
    echo " **** Output file (binary format): "autoboot.bin
    srec_cat $1.sr -Motorola -o out_win.sr -Motorola -address-length=3 -CRLF
    srec_cat $1.sr -Motorola -o out.sr -Motorola -address-length=3
    echo " **** Output file (s-record format - LF EOL): "out.sr
    echo " **** Output file (s-record format - CRLF EOL): "out_win.sr
else
    echo " **** Error compiling: "$1.c " - Aborted"
fi
